/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Tab Ctrl Viewer.
 *
 * The Initial Developer of the Original Code is
 * Emil Hesslow.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

addEventListener("load", onLoad, false);
addEventListener("unload", onUnload, false);

function onLoad()
{
  ctrlTabPreview.init();
}

function onUnload()
{
  ctrlTabPreview.destruct();
}


var ctrlTabPreview = {
  mViewing:           0,
  mMac:               false,
  mSelectedTabIndex: -1,
  mWindowInfo:           []
,
  init: function()
  {
    this.mac = (navigator.platform.match(/mac/i)) ? true : false;
    addEventListener("keydown",  function(aEvent) { ctrlTabPreview.keydown(aEvent); },   true);
    addEventListener("keyup",    function(aEvent) { ctrlTabPreview.keyup(aEvent); },     true);
    addEventListener("keypress", function(aEvent) { ctrlTabPreview.keypress(aEvent); },  true);
  }
,
  destruct: function()
  {
    removeEventListener("keydown",  function(aEvent) { ctrlTabPreview.keydown(aEvent); },   true);
    removeEventListener("keyup",    function(aEvent) { ctrlTabPreview.keyup(aEvent); },     true);
    removeEventListener("keypress", function(aEvent) { ctrlTabPreview.keypress(aEvent); },  true);
  }
,
  view: function(aDir)
  {
    this.mViewing++;

    var screenManager = Components.classes["@mozilla.org/gfx/screenmanager;1"].getService(Components.interfaces.nsIScreenManager);
    var screen = screenManager.screenForRect(window.screenX, window.screenY, window.outerWidth, window.outerHeight);

    var left = {}, top = {}, width = {}, height = {};
    screen.GetAvailRect(left, top, width, height);

    var w = gBrowser.contentWindow.innerWidth;
    var h = gBrowser.contentWindow.innerHeight;

    var smallWidth = Math.round(width.value/4);
    var smallHeight = Math.round(smallWidth/w*h);
    var largeWidth = Math.round(width.value/3);
    var largeHeight = Math.round(largeWidth/w*h);

    var browser = getBrowser();
    var tabs = browser.tabContainer;

    var popup = [
                  ["previewPopupLeft",
                   "previewCanvasLeft",
                   smallWidth,
                   smallHeight,
                   Math.round(left.value + (width.value/3-smallWidth)/2),
                   Math.round(top.value + (height.value-smallHeight)/2),
                   "previewLabelLeft",
                   "previewLabelLeftNumber",
                   "previewLabelBoxLeft"
                  ],
                  ["previewPopupCenter",
                   "previewCanvasCenter",
                   largeWidth,
                   largeHeight,
                   Math.round(left.value + width.value/3),
                   Math.round(top.value + (height.value-largeHeight)/2),
                   "previewLabelCenter",
                   "previewLabelCenterNumber",
                   "previewLabelBoxCenter"
                  ],
                  ["previewPopupRight",
                   "previewCanvasRight",
                   smallWidth,
                   smallHeight,
                   Math.round(left.value + (width.value/3*5-smallWidth)/2),
                   Math.round(top.value + (height.value-smallHeight)/2),
                   "previewLabelRight",
                   "previewLabelRightNumber",
                   "previewLabelBoxRight"
                  ]
                 ];

    var i = 0;
    var tabIndex = [this.mSelectedTabIndex];
    if (tabs.childNodes.length == 1)
    {
      i = 1;
      document.getElementById(popup[0][0]).hidePopup();
      document.getElementById(popup[2][0]).hidePopup();
    }
    else
    {
      if (this.mSelectedTabIndex >= tabs.childNodes.length-1)
        tabIndex.push(0);
      else
        tabIndex.push(this.mSelectedTabIndex+1);

      if (this.mSelectedTabIndex == 0)
        tabIndex.unshift(tabs.childNodes.length-1);
      else
        tabIndex.unshift(this.mSelectedTabIndex-1);
    }

    this.mWindowInfo = [];
    for (var j = 0; j < tabIndex.length; j++)
    {
      var t = tabIndex[j];
      if (tabs.childNodes[t] && tabs.childNodes[t].boxObject.firstChild)
      {
        document.getElementById(popup[i][0]).showPopup(document.documentElement , popup[i][4], popup[i][5], "popup", null, null);

        cBrowser = browser.getBrowserForTab(tabs.childNodes[t]);
        var cWindow = cBrowser.contentWindow;

        var scrollWidth = 0;
        if (cWindow.scrollMaxY > 0)
          scrollWidth = 20;

        document.getElementById(popup[i][6]).value = cBrowser.contentDocument.title;
        document.getElementById(popup[i][7]).value = "#"+ (t+1);
        document.getElementById(popup[i][8]).style.maxWidth = popup[i][2] +"px";

        var canvas = document.getElementById(popup[i][1]);
        canvas.style.width = popup[i][2] +"px";
        canvas.style.height = popup[i][3] +"px";
        canvas.width = popup[i][2];
        canvas.height = popup[i][3];

        var ctx = canvas.getContext("2d");
        if (this.mViewing <= 1)
        {
          ctx.clearRect(0, 0, popup[i][2], popup[i][3]);
          ctx.fillStyle = "rgb(255,255,255)";
          ctx.fillRect(0, 0, popup[i][2], popup[i][3]);
          ctx.save();
        }

        this.mWindowInfo.push([popup[i][1], cWindow, w-scrollWidth, h, popup[i][2], popup[i][3]]);

        if (this.mViewing > 1)
          this.loadPreview();
        else
          setTimeout("ctrlTabPreview.loadPreview();", 1);
      }
      i++;
    }
  }
,
  loadPreview: function()
  {
    if (this.mWindowInfo.length > 0)
    {
      var windowInfo = this.mWindowInfo.shift();
      var canvas = document.getElementById(windowInfo[0]);
      var ctx = canvas.getContext("2d");
      ctx.clearRect(0, 0, windowInfo[4], windowInfo[5]);
      ctx.save();
      ctx.scale(windowInfo[4]/windowInfo[2], windowInfo[5]/windowInfo[3]);
      ctx.drawWindow(windowInfo[1], windowInfo[1].scrollX, windowInfo[1].scrollY, windowInfo[1].scrollX+windowInfo[2], windowInfo[1].scrollY+windowInfo[3], "rgb(255,255,255)");
      ctx.restore();

      setTimeout("ctrlTabPreview.loadPreview();", 1);
    }
  }
,
  hide: function()
  {
    this.mViewing = 0;

    document.getElementById("previewPopupLeft").hidePopup();
    document.getElementById("previewPopupCenter").hidePopup();
    document.getElementById("previewPopupRight").hidePopup();
  }
,
  // If we press Ctrl+Tab show the preview
  keydown: function(aEvent)
  {
    if (aEvent.keyCode == Components.interfaces.nsIDOMKeyEvent.DOM_VK_TAB && (aEvent.ctrlKey && !this.mac || aEvent.metaKey && this.mac))
    {
      var browser = getBrowser();
      var tabs = browser.tabContainer;

      if (this.mSelectedTabIndex < 0)
        this.mSelectedTabIndex = tabs.selectedIndex;

      if (aEvent.shiftKey)
        this.mSelectedTabIndex += tabs.childNodes.length-1;
      else
        this.mSelectedTabIndex++;
      this.mSelectedTabIndex = this.mSelectedTabIndex % tabs.childNodes.length;

      this.view();

      aEvent.preventDefault();
      aEvent.stopPropagation();
    }
  }
,
  // When we release Ctrl we are ready
  keyup: function(aEvent)
  {
    if ((aEvent.keyCode == Components.interfaces.nsIDOMKeyEvent.DOM_VK_CONTROL && !this.mac || aEvent.keyCode == Components.interfaces.nsIDOMKeyEvent.DOM_VK_META && this.mac) && this.mViewing > 0)
    {
      var browser = getBrowser();
      var tabs = browser.tabContainer;
      tabs.selectedIndex = this.mSelectedTabIndex;
      
      this.mSelectedTabIndex = -1;
      this.hide();
    }
  }
,
  // Used for disable the default behavior
  keypress: function(aEvent)
  {
    if (aEvent.keyCode == Components.interfaces.nsIDOMKeyEvent.DOM_VK_TAB && (aEvent.ctrlKey && !this.mac || aEvent.metaKey && this.mac))
    {
      aEvent.preventDefault();
      aEvent.stopPropagation();
    }
  }
}
